﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;
using StyleCop;
using StyleCop.CSharp;

namespace WithSettingRule
{
    [SourceAnalyzer(typeof(CsParser))]
    public class MyDocRule : SourceAnalyzer
    {
        [StructLayout(LayoutKind.Sequential)]
        private struct AnalyzerSettings
        {
            public bool IncludePublic;
            public bool IncludeInternal;
        }


        private readonly string headerSummaryForGetAndSetAccessor = "可读写";
        private readonly string headerSummaryForGetAccessor = "只读";
        private readonly string headerSummaryForSetAccessor = "只写";

        private void ParseHeader(CsElement element, XmlHeader header, int lineNumber, bool partialElement)
        {
            XmlDocument doc = this.LoadHeaderIntoDocument(element, header, lineNumber);
            if ((doc != null) && (element.ElementType == ElementType.Property))
            {
                this.CheckPropertySummaryFormatting(element as Property, doc);
            }
        }

        private XmlDocument LoadHeaderIntoDocument(CsElement element, XmlHeader header, int lineNumber)
        {
            XmlDocument document = new XmlDocument();
            try
            {
                string xml = "<root>" + header.Text + "</root>";
                document.LoadXml(xml);
            }
            catch (XmlException exception)
            {
                base.AddViolation(element, lineNumber, Rules.DocumentationMustContainValidXml, new object[] { exception.Message });
                document = null;
            }
            catch (Exception exception)
            {
                base.AddViolation(element, lineNumber, Rules.DocumentationMustContainValidXml, new object[] { exception.Message });
                document = null;
            }
            return document;
        }

        private void CheckPropertySummaryFormatting(Property property, XmlDocument doc)
        {
            XmlNode node = doc.SelectSingleNode("root/summary");
            if (node != null)
            {
                bool flag = ((property.ReturnType.Text == "bool") || (property.ReturnType.Text == "Boolean")) || (property.ReturnType.Text == "System.Boolean");
                if (property.GetAccessor != null)
                {
                    if ((property.SetAccessor == null) || ((property.SetAccessor.AccessModifier != property.AccessModifier) && (((property.AccessModifier == AccessModifierType.Private) || (property.SetAccessor.AccessModifier != AccessModifierType.Private)) || property.SetAccessor.Declaration.ContainsModifier(new CsTokenType[] { CsTokenType.Private }))))
                    {
                        string str2 = headerSummaryForGetAccessor;
                        string str3 = node.InnerText.TrimStart(new char[0]);
                        if (!CompareTextWithColon(str3, str2))
                        {
                            base.AddViolation(property, Rules.PropertySummaryDocumentationMustMatchAccessors, new object[] { str2 });
                        }
                    }
                    else
                    {
                        string str = headerSummaryForGetAndSetAccessor;
                        if (!CompareTextWithColon(node.InnerText.TrimStart(new char[0]), str))
                        {
                            base.AddViolation(property, Rules.PropertySummaryDocumentationMustMatchAccessors, new object[] { str });
                        }
                    }
                }
                else if (property.SetAccessor != null)
                {
                    string str5 = headerSummaryForSetAccessor;
                    if (!CompareTextWithColon(node.InnerText.TrimStart(new char[0]), str5))
                    {
                        base.AddViolation(property, Rules.PropertySummaryDocumentationMustMatchAccessors, new object[] { str5 });
                    }
                }
            }
        }

        private bool CompareTextWithColon(string input, string pattern)
        {
            return Regex.IsMatch(input, pattern + "[:：]", RegexOptions.Singleline);
        }

        private bool CheckDocumentationForElement(CsElement element, CsElement parentElement, AnalyzerSettings settings)
        {
            if (base.Cancel)
            {
                return false;
            }

            if (!element.Generated)
            {
                if ((element.ElementType == ElementType.Property) &&
                ((settings.IncludePublic && element.AccessModifier == AccessModifierType.Public) ||
                   (settings.IncludeInternal && element.AccessModifier == AccessModifierType.Internal)))
                {
                    this.ParseHeader(element, element.Header, element.LineNumber, false);
                }
            }
            return true;
        }

        public override void AnalyzeDocument(CodeDocument document)
        {
            Param.RequireNotNull(document, "document");
            CsDocument document2 = (CsDocument)document;
            if ((document2.RootElement != null) && !document2.RootElement.Generated)
            {
                AnalyzerSettings settings = LoadSettings(document);

                document2.WalkDocument<AnalyzerSettings>(new CodeWalkerElementVisitor<AnalyzerSettings>(this.CheckDocumentationForElement), settings);
            }
        }

        private AnalyzerSettings LoadSettings(CodeDocument document)
        {
            AnalyzerSettings result = new AnalyzerSettings();
            result.IncludeInternal = true;
            result.IncludePublic = true;

            if (document.Settings != null)
            {
                BooleanProperty addInSetting = document.Settings.GetAddInSetting(this, "IncludePublic") as BooleanProperty;
                if (addInSetting != null)
                {
                    result.IncludePublic = addInSetting.Value;
                }

                addInSetting = document.Settings.GetAddInSetting(this, "IncludeInternal") as BooleanProperty;
                if (addInSetting != null)
                {
                    result.IncludeInternal = addInSetting.Value;
                }
            }

            return result;
        }
    }
}
